/**
 * Header file to the "koolplot" library
 * 
 * Provides basic, but also very simple-to-use 2D data plotting for
 * C++ programs.
 *
 * Example:
 * #include <koolplot.h>
 * int main()
 * {
 * 		Plotdata<> x(0, 360), y = sin(x * M_PI / 180);
 *		plot(x, y);
 *		return 0;
 * }
 *
 * The "plot" function opens a window and displays the graph.
 * The next plot is shown when SPACE key is pressed.
 *
 * Author: 	jlk
 * Version:	1.1
 * Date:	July 2005
 */
 
#pragma once

#include "plotstream.cpp"

/** Plot x..y data, specify label */
template<class T> void plot(const Plotdata<T>&x,const Plotdata<T>&y,const char*label=0) {
 Plotstream<T> ps(label); ps.plot(x, y);
} 

/** Plot x..y data, specify curve colour, then label */
template<class T> void plot(const Plotdata<T>&x,const Plotdata<T>&y,Color colour,const char*label=0) {
 Plotstream<T> ps(label); ps.plot(x, y, colour);
} 

/** Plot x..y data, specify label and curve colour */
template<class T> inline void plot(const Plotdata<T>&x, const Plotdata<T>&y, const char*l, int c) {plot(x,y,c,l);}


/** Plot f(x) function, specify label (use together with f(x) function) */
template<class T> void plot(const Plotdata<T>&x,Func<T>&f,const char*label=0) {
 if (!f) return;
 Plotstream<T> ps(label); ps.plot(x, x.doFunc(f));
} 

/** Plot f(x) function, specify curve colour, then label (use together with f(x) function) */
template<class T> void plot(const Plotdata<T>&x,Func<T>&f,Color colour,const char*label=0) {
 if (!f) return;
 Plotstream<T> ps(label); ps.plot(x, x.doFunc(f), colour);
} 

/** Plot f(x) function, specify label and curve colour (use together with f(x) function) */
template<class T> inline void plot(const Plotdata<T>&x,Func<T>&f,const char *l,Color c) {plot(x,f,c,l);}

// Installs a user-defined unary function of x as f(x)
// example: Plotdata x(-6, 6);
//		f(x) = sinc;	// if sinc is a function defined by the user.
//		plot(x, f(x));	// Plots the graph of sinc(x) from -6 to 6
template<class T> Func<T>&f(Plotdata<T>&x) {return x.userfunc();}

// Installs a user-defined binary function of x as f2(x)
template<class T> BinFunc<T>&f2(Plotdata<T>&x) {return x.userBinfunc();}

// Calculates a user-defined binary function of x as y = f2(x, double_value)
// example: Plotdata x(-270, 270);
//		f2(x) = tanLessThan; //  tanLessThan -user-defined binary function.
//		plot(x, f2(x, 10));	 // Plots the graph of tanLessThan(x, 10)
template<class T> Plotdata<T> f2(const Plotdata<T>&x, T op2) {return x.doBinFunc(op2);}

/**Adds next point to plotting data
 * -You add x and y coordinates in 2 separate Plotdatas then call plot(). 
 * plot() will draw the curve joining all the data points. 
 * 
 * @param pd  the Plotdata to add the point to
 * @param val value of the point to add */
template<class T> void point(Plotdata<T>&pd, T val) {pd.point(val);}

/**Adds next point to plotting data
 *  
 * Same as above, but will take both axes and coordinates in one call. 
 * 
 * @param xAxis the x Plotdata to add the point to
 * @param xCoord value of the x coordinate to add
 * @param yAxis the y Plotdata to add the point to
 * @param yCoord value of the y coordinate to add */
template<class T> void point(Plotdata<T>&xAxis, Plotdata<T>&yAxis,T xCoord, T yCoord) {
 xAxis.point(xCoord); yAxis.point(yCoord);
}

/**Insert array of data points into plotting data
 *  
 * @param axis the  Plotdata to add the points to
 * @param array The array of points to insert
 * @param numToInsert the number of points to add from the array. */
template<class T> void insert(Plotdata<T>& axis, const T ary[], int numToInsert) {
 axis.insert(ary, numToInsert);
}

// Clear previous data from an axis.
template<class T> void clear(Plotdata<T>&pd) {pd.clear();}

/*________ Maths functions that may be used to define functions ______*/

template<class T> Plotdata<T> sin(const Plotdata<T>&pd) {return pd.doFunc(sin);}
template<class T> Plotdata<T> cos(const Plotdata<T>&pd) {return pd.doFunc(cos);}
template<class T> Plotdata<T> tan(const Plotdata<T>&pd) {return pd.doFunc(tan);}
template<class T> Plotdata<T> asin(const Plotdata<T>&pd) {return pd.doFunc(asin);}
template<class T> Plotdata<T> acos(const Plotdata<T>&pd) {return pd.doFunc(acos);}
template<class T> Plotdata<T> atan(const Plotdata<T>&pd) {return pd.doFunc(atan);}
template<class T> Plotdata<T> sinh(const Plotdata<T>&pd) {return pd.doFunc(sinh);}
template<class T> Plotdata<T> cosh(const Plotdata<T>&pd) {return pd.doFunc(cosh);}
template<class T> Plotdata<T> tanh(const Plotdata<T>&pd) {return pd.doFunc(tanh);}
template<class T> Plotdata<T> sqrt(const Plotdata<T>&pd) {return pd.doFunc(sqrt);}
template<class T> Plotdata<T> fabs(const Plotdata<T>&pd) {return pd.doFunc(fabs);}
template<class T> Plotdata<T> log(const Plotdata<T>&pd) {return pd.doFunc(log);}
template<class T> Plotdata<T> log10(const Plotdata<T>&pd) {return pd.doFunc(log10);}
template<class T> Plotdata<T> exp(const Plotdata<T>&pd) {return pd.doFunc(exp);}
template<class T> Plotdata<T> pow(const Plotdata<T>&pd, T exp) {return pd.doBinFunc(pow,exp);}
